import json
import operator
import os
from datetime import datetime
from django.conf import settings

import collections

from .exceptions import ACPCalendarException

from . import app_settings

def load_data(apps, schema_editor):
    HolidayType = apps.get_model("acp_calendar", "HolidayType")
    for holiday_type in get_holiday_type_list():
        HolidayType.objects.create(**holiday_type)

    ACPHoliday = apps.get_model("acp_calendar", "ACPHoliday")
    for holiday_data in get_holidays_list():
        try:
            holiday_type = HolidayType.objects.get(short_name=holiday_data['holiday_type'])
            holiday_date = datetime.strptime(holiday_data['date'], app_settings.LOAD_DATE_FORMAT)
            ACPHoliday.objects.create(date=holiday_date, holiday_type=holiday_type )
        except HolidayType.DoesNotExist:
            raise ACPCalendarException('Could not find a holiday type for %s' % holiday_data['holiday_type'])


def get_holiday_type_list():
    holiday_types = [{'name': 'Año Nuevo', 'short_name': 'año_nuevo'},
                     {'name': 'Día de los Mártires', 'short_name': 'mártires'},
                     {'name': 'Martes Carnaval', 'short_name': 'martes_carnaval'},
                     {'name': 'Viernes Santo', 'short_name': 'viernes_santo'},
                     {'name': 'Día del Trabajador', 'short_name': 'día_del_trabajo'},
                     {'name': 'Toma de Posesión Presidencial', 'short_name': 'toma_presidencial'},
                     {'name': 'Día de la Separación de Panamá de Colombia', 'short_name': 'separación_colombia'},
                     {'name': 'Día de Colón', 'short_name': 'colón'},
                     {'name': 'Primer Grito de Independencia', 'short_name': 'grito_independencia'},
                     {'name': 'Independencia de Panamá de España', 'short_name': 'independencia_españa'},
                     {'name': 'Día de la Madre', 'short_name': 'día_de_la_madre'},
                     {'name': 'Navidad', 'short_name': 'navidad'},
                     ]
    return holiday_types


def get_holidays_dictionary():
    holiday_list = get_holidays_list()
    holiday_dictionary = dict()
    for holiday_data in holiday_list:
        year = holiday_data['date'][:4]
        if year not in holiday_dictionary.keys():
            holiday_dictionary[year] = list()
        holiday_dictionary[year].append(holiday_data)
    ordered_holidays = collections.OrderedDict(sorted(holiday_dictionary.items()))
    return ordered_holidays


def get_key(object):
    return object['date']


def get_holidays_list(source_json_file = None):
    if source_json_file is None:
        data_filename = app_settings.INITIAL_DATA_FILENAME
    elif source_json_file is not None and settings.DEBUG:
        data_filename = source_json_file
    else:
        raise ValueError('Cannot change json source')
    with open(data_filename, encoding='utf-8') as json_data:
        holidays_list = json.load(json_data)
    ordered_holidays_list = sorted(holidays_list, key=operator.itemgetter('date'))
    return ordered_holidays_list



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations

from ..initial_data import load_data


class Migration(migrations.Migration):

    dependencies = [
        ('acp_calendar', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(load_data)
    ]



import json

from django.db import models
from . import app_settings


class ACPHolidayQuerySet(models.QuerySet):

    def write_json(self, filename):
        holidays = list()
        db_holidays = self
        for db_holiday in db_holidays:
            holiday_dict = dict()
            holiday_dict['date'] = db_holiday.date.strftime(app_settings.LOAD_DATE_FORMAT)
            holiday_dict['holiday_type'] = db_holiday.holiday_type.short_name
            holidays.append(holiday_dict)

        with open(filename, 'w', encoding='utf-8') as outfile:
            json.dump(holidays, outfile, indent=4, ensure_ascii=False)

        return self


class ACPHolidayManager(models.Manager):

    def get_queryset(self):
        return ACPHolidayQuerySet(self.model, using=self._db)

    def update_fiscal_years(self):
        from .models import FiscalYear
        holidays_without_fiscal_year = self.filter(fiscal_year=0)
        for holiday in holidays_without_fiscal_year:
            fy = FiscalYear.create_from_date(holiday.date)
            holiday.fiscal_year = fy.year
            holiday.save()
        return holidays_without_fiscal_year



from django.contrib import admin

from .models import HolidayType, ACPHoliday


class HolidayTypeAdmin(admin.ModelAdmin):
    list_display = ('short_name', 'name')

class ACPHolidayAdmin(admin.ModelAdmin):
    list_display = ('date', 'holiday_type', 'fiscal_year')


admin.site.register(HolidayType, HolidayTypeAdmin)
admin.site.register(ACPHoliday, ACPHolidayAdmin)



from rest_framework import serializers
from rest_framework.pagination import PageNumberPagination

from ..models import HolidayType, ACPHoliday


class StandardResultsSetPagination(PageNumberPagination):
    page_size = 20
    page_size_query_param = 'page_size'
    max_page_size = 1000

class HolidayTypeSerializer(serializers.ModelSerializer):

    class Meta:
        model = HolidayType
        fields = ('id', 'name')


class ACPHolidaySerializer(serializers.ModelSerializer):

    class Meta:
        model = ACPHoliday
        fields = ('id', 'date', 'holiday_type')



from datetime import date
from django.contrib import messages
from django.db.models import Count
from django.shortcuts import render
from django.views.generic import View
from django.utils import timezone

from .utils import compare_initial_data_against_db
from . import __version__ as current_version
from .exceptions import ACPCalendarException
from .models import ACPHoliday, FiscalYear
from \
    .forms import CalculatorForm


class HomeView(View):

    template_name = 'acp_calendar/home.html'

    def get(self, request, *args, **kwargs):
        data = self._build_data_dict()
        return render(request, self.template_name, data)

    def post(self, request, *args, **kwargs):
        if 'update_fiscal_year' in request.POST:
            ACPHoliday.objects.update_fiscal_years()
            data = self._build_data_dict()
        elif 'check_initial_data' in request.POST:
            not_found = compare_initial_data_against_db()
            data = self._build_data_dict()
            data['not_found'] = not_found
        return render(request, self.template_name, data)

    def _build_data_dict(self):
        data = dict()
        data['first_holiday'] = ACPHoliday.objects.first()
        data['last_holiday'] = ACPHoliday.objects.last()
        data['holiday_count'] = ACPHoliday.objects.count()
        data['version'] = current_version
        #OrderNotes.objects.filter(item=item).values_list('shared_note', flat=True).distinct()
        #data['years'] = ACPHoliday.objects.order_by('-fiscal_year').distinct('fiscal_year').values('fiscal_year')
        data['years'] = ACPHoliday.objects.exclude(fiscal_year=0).order_by('-fiscal_year').values_list('fiscal_year', flat=True).distinct()
        # if len(data['years']) == 1 and data['years'][0] == 0:
        #     data['years'] = []
        return data


class CalendarView(View):
    """
    View to generate a calendar for fiscal year containg the working days in every month
    of the fiscal year.
    """

    template_name = 'acp_calendar/fiscal_year_calendar.html'

    def get(self, request, *args, **kwargs):
        year = int(kwargs['fiscal_year'])
        fiscal_year = FiscalYear(year)
        data = dict()
        data['months'] = list()
        data['version'] = current_version
        data['fiscal_year'] = year
        data['working_days_in_fiscal_year'] = 0
        try:
            for month in fiscal_year.months_in_fiscal_year():
                month_data = dict()
                month_data['month'] = date(month[1], month[0], 1).strftime('%b')
                month_data['year'] = month[1]
                month_data['working_days'] = ACPHoliday.get_working_days_for_month(month[1], month[0])
                data['working_days_in_fiscal_year'] += month_data['working_days']
                data['months'].append(month_data)
            today = timezone.now().date()
            if today <= fiscal_year.end_date:
                data['remaining_working_days_in_fiscal_year'] = ACPHoliday.get_working_days(today, fiscal_year.end_date)
            else:
                data['remaining_working_days_in_fiscal_year'] = 0

            data['remaining_working_days_percentage'] = data['remaining_working_days_in_fiscal_year'] / data['working_days_in_fiscal_year'] *100
        except ACPCalendarException as e:
            data['errors'] = str(e)
        return render(request, self.template_name, data)


class CalculatorView(View):
    """
    View to calculate the amount of working day between two dates.
    """

    template_name = 'acp_calendar/calculator.html'

    def get(self, request, *args, **kwargs):
        form = CalculatorForm()
        data = {'form': form,
                'version': current_version}
        return render(request, self.template_name, data)

    def post(self, request, *args, **kwargs):
        calculator_form = CalculatorForm(request.POST)
        data = {'form': calculator_form,
                'working_days': None,
                'version': current_version}
        if calculator_form.is_valid():
            start_date = calculator_form.cleaned_data['start_date']
            end_date = calculator_form.cleaned_data['end_date']
            try:
                working_days = ACPHoliday.get_working_days(start_date, end_date)
                data['working_days'] = working_days
                return render(request, self.template_name, data)
            except ACPCalendarException as e:
                messages.add_message(request, messages.ERROR, str(e), extra_tags='dragonball')
                return render(request, self.template_name, data)
        else:
            return render(request, self.template_name, data)




# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-11-27 13:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('acp_calendar', '0002_load_holiday_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='acpholiday',
            name='fiscal_year',
            field=models.IntegerField(default=0, verbose_name='fiscal year'),
        ),
    ]



from django.conf.urls import url
from . import views

urlpatterns = [
    url(r"^working-days/(?P<start_date>[0-9]{4}-[0-9]{2}-[0-9]{2})/(?P<end_date>[0-9]{4}-[0-9]{2}-[0-9]{2})/$",
        views.CalendarCalculationsView.as_view(), {'calculation': 'working_days'}, name="working_days", ),
    url(r"^working-delta/(?P<start_date>[0-9]{4}-[0-9]{2}-[0-9]{2})/(?P<days>[0-9]+)/$",
        views.CalendarCalculationsView.as_view(), {'calculation': 'working_delta'},name="working_delta"),
    url(r"^working-month/(?P<year>[0-9]{4})/(?P<month>[0-9]{1,2})/$",
        views.CalendarCalculationsView.as_view(),{'calculation': 'working_month'}, name="working_month"),
    url(r'^holiday-list/$', views.ACPHolidayListAPIView.as_view(), name='holiday-list'),
    url(r'^holiday-list/(?P<year>[0-9]{4})/$', views.ACPHolidayListAPIView.as_view(), name='holiday-list-year'),
    ]



from . import app_settings
from .models import ACPHoliday
from .initial_data import get_holidays_list
from datetime import datetime
from . import app_settings

def compare_initial_data_against_db(initial_data_json=None):
    """
    This method compares the holidays in holiday_initial_data.json to the content of the database
    :param initial_data_json: String with filename of json file
    :return: list of dictionaries containing date and holiday_type of holidays that are not the database
    """
    holidays = get_holidays_list(initial_data_json)
    not_found = list()
    for holiday in holidays:
        try:
            ACPHoliday.objects.get(date=datetime.strptime(holiday['date'], app_settings.LOAD_DATE_FORMAT))
        except ACPHoliday.DoesNotExist:
            not_found.append(holiday)
    return not_found




from django.conf.urls import url, include
from django.views.generic import TemplateView

from .views import CalculatorView, CalendarView, HomeView

urlpatterns = [
    url(r'^calculator/',  CalculatorView.as_view(), name='calculator'),
    url(r'^$',  HomeView.as_view(), name='home'),
    url(r'^fiscal-year-calendar/(?P<fiscal_year>\d{4})',  CalendarView.as_view(), name='fiscal-year-calendar'),
    url(r'^api/', include('acp_calendar.api.urls', namespace='calendar-api')),
    ]
